/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <polygon.h>

#include "optset.h"
#include "errdesc.h"




OptionSet emptyoptset;



OptionSet::OptionSet()
{
  setDefaults();
}


/*OptionSet::OptionSet(Parser &p)
{
  setDefaults();
  read(p);
  }*/

void OptionSet::setDefaults(void)
{
  create_ncp = create_w = true;
  surface=0;
  tedelete=0;
  recolor_blue=false;
  transparency = OS_NO_TRANS;
}

OptionSet::~OptionSet()
{}



int OptionSet::dummyread(Parser &p)
{
  int level=0;

  if(p.curtok == PT_BBEGIN)
    {
      do
	{
	  if(p.curtok==PT_BBEGIN)
	    level++;
	  else if(p.curtok == PT_BEND)
	    level--;
	  else if(p.curtok == PT_TEXT)
	    lprintf(3,"    skip %s\n",p.tok_text);
	  p.gentoken();
	}
      while ( (p.curtok != PT_EOF) && (level > 0) );

    };
  return 0;
}


int OptionSet::setTrans(Parser &p)
{
  int val;
  char *cur=p.tok_text;

  while(*cur != 0)
    {
      if((*cur<'0') || (*cur>'9'))
	{
	  p.error(ERROR_EXPECTED_KEY,"Transparency value must be a number, not '%s'",p.tok_text);
	  return 100;
	};
      cur++;
    };

  sscanf(p.tok_text,"%d",&val);
  if((val<0)||(val>255))
    {
      p.error(ERROR_EXPECTED_KEY,"Transparency value must be between 0 and 255");
      return 100;
    };

  transparency=val;
  return 0;
}


int OptionSet::setTEColor(Parser &p)
{
  rvulong val;
  char *cur=p.tok_text;

  if(strlen(p.tok_text)!= 6)
    {
      p.error(ERROR_EXPECTED_KEY,"TE-Color value must be a 6-digit hex triplet, not '%s'",p.tok_text);
      return 100;
    };

  while(*cur != 0)
    {
      if(! (   ((*cur>='0') && (*cur<='9'))
	     ||((*cur>='a') && (*cur<='f'))))
	{
	  p.error(ERROR_EXPECTED_KEY,"TE-Color value must be a hex triplet, not '%s'",p.tok_text);
	  return 100;
	};
      cur++;
    };

  sscanf(p.tok_text,"%x",&val);
  if(val>0xffffff)
    {
      p.error(ERROR_EXPECTED_KEY,"TE-Color value must be between 000000 and ffffff");
      return 100;
    };

  bluecolor=val;
  recolor_blue=true;
  return 0;
}


int OptionSet::parseTEColor(Parser &p)
{
  int res=0;

  if(p.curtok == PT_BBEGIN)
    {
      p.gentoken();
      if(p.curtok == PT_TEXT)
	{
	  res=setTEColor(p);
	  p.gentoken();
	}
      else
	{
	  p.error(ERROR_EXPECTED_KEY,"RGB triplet in 'tecolor' section expected.");
	  return 100;
	};

      if(p.curtok == PT_BEND)
	p.gentoken();
      else
	{
	  p.error(ERROR_BEND_EXPECTED,"Trailing garbage in 'tecolor' section");
	  return 100;
	};
    };
  return res;
}

int OptionSet::setSurface(Parser &p)
{
  char *cur;

  if(strcasecmp(p.tok_text,"standard")==0)
    {
      surface=0;
      return 0;
    }
  else if(strcasecmp(p.tok_text,"grass")==0)
    {
      surface=4;
      return 0;
    }
  else if(strcasecmp(p.tok_text,"notracks")==0)
    {
      surface=8;
      return 0;
    }
  else if(strcasecmp(p.tok_text,"mud")==0)
    {
      surface=18;
      return 0;
    }
  else if(strcasecmp(p.tok_text,"slippery")==0)
    {
      surface=21;
      return 0;
    };

  cur=p.tok_text;
  while(*cur != 0)
    {
      if((*cur<'0') || (*cur>'9'))
	{
	  p.error(ERROR_EXPECTED_KEY,"Surface type must be a number or surface keyword, not '%s'",p.tok_text);
	  return 100;
	};
      cur++;
    };

  sscanf(p.tok_text,"%d",&surface); /* We allow any number here! */

  return 0;
}


int OptionSet::parseSurface(Parser &p)
{
  int res=0;

  if(p.curtok == PT_BBEGIN)
    {
      p.gentoken();
      if(p.curtok == PT_TEXT)
	{
	  res=setSurface(p);
	  p.gentoken();
	}
      else
	{
	  p.error(ERROR_EXPECTED_KEY,"Surface keyword or number in 'surface' section expected.");
	  return 100;
	};

      if(p.curtok == PT_BEND)
	p.gentoken();
      else
	{
	  p.error(ERROR_BEND_EXPECTED,"Trailing garbage in 'surface' section");
	  return 100;
	};
    };
  return res;
}

int OptionSet::parseTrans(Parser &p)
{
  int res=0;

  if(p.curtok == PT_BBEGIN)
    {
      p.gentoken();
      if(p.curtok == PT_TEXT)
	{
	  res=setTrans(p);
	  p.gentoken();
	}
      else
	{
	  p.error(ERROR_EXPECTED_KEY,"Transparency value in 'transparency' section expected.");
	  return 100;
	};

      if(p.curtok == PT_BEND)
	p.gentoken();
      else
	{
	  p.error(ERROR_BEND_EXPECTED,"Trailing garbage in 'transparency' section");
	  return 100;
	};
    };
  return res;
}

int OptionSet::setTEDelete(Parser &p)
{
  if(strcasecmp(p.tok_text,"pwall")==0)
    tedelete |= TEP_PWALL;
  else if(strcasecmp(p.tok_text,"rwall")==0)
    tedelete |= TEP_RWALL;
  else if(strcasecmp(p.tok_text,"carimg")==0)
    tedelete |= TEP_CARIMG;
  else if(strcasecmp(p.tok_text,"floor")==0)
    tedelete |= TEP_FLOOR;
  else if(strcasecmp(p.tok_text,"lamp")==0)
    tedelete |= TEP_LAMP;
  else if(strcasecmp(p.tok_text,"box")==0)
    tedelete |= TEP_BOX;
  else if(strcasecmp(p.tok_text,"track")==0)
    tedelete |= TEP_TRACK;
  else
    {
      p.error(ERROR_EXPECTED_TED_KEY,"Expected keyword for 'tedelete' section, not '%s'",p.tok_text);
      return 100;
    };
  return 0;
}

int OptionSet::parseTEDelete(Parser &p)
{
  int res=0;

  if(p.curtok == PT_BBEGIN)
    {
      do
	{
	  p.gentoken();
	  if(p.curtok == PT_TEXT)
	    {
	      res=setTEDelete(p);
	      if(res != 0)
		return res;
	    }
	  else if(p.curtok == PT_EOF)
	    {
	      p.error(ERROR_EOF_IN_BLOCK,"End of file reached while reading 'tedelete' section");
	      return 100;
	    }
	  else if(p.curtok != PT_BEND)
	    {
	      p.error(ERROR_BEND_EXPECTED,"Trailing garbage in 'transparency' section");
	      return 100;
	    };
	}
      while(p.curtok != PT_BEND);
      p.gentoken();
    };
  return res;
}

int OptionSet::parseItem(Parser &p)
{
  if(strcasecmp(p.tok_text,"tecolor")==0)
    {
      p.gentoken();
      return parseTEColor(p);
    }
  else if(strcasecmp(p.tok_text,"invisible")==0)
    {
      create_w=false;
      p.gentoken();
    }
  else if(strcasecmp(p.tok_text,"tedelete")==0)
    {
      p.gentoken();
      return parseTEDelete(p);
    }
  else if(strcasecmp(p.tok_text,"nocoll")==0)
    {
      create_ncp=false;
      p.gentoken();
    }
  else if(strcasecmp(p.tok_text,"off")==0)
    {
      create_w=create_ncp=false;
      p.gentoken();
    }
  else if((strcasecmp(p.tok_text,"surface")==0)||(strcasecmp(p.tok_text,"s")==0))
    {
      p.gentoken();
      return parseSurface(p);
    }
  else if((strcasecmp(p.tok_text,"transparency")==0)||(strcasecmp(p.tok_text,"t")==0))
    {
      p.gentoken();
      return parseTrans(p);
    }
  else
    {
      p.error(ERROR_UNKNOWN_OPTION,"Unknown option '%s'",p.tok_text);
      return 100;
    };
  return 0;
}


int OptionSet::read(Parser &p)
{
  int res;

  if(p.curtok != PT_BBEGIN)
    return 0;

  p.gentoken();
  while(p.curtok != PT_BEND)
    {
      if(p.curtok == PT_EOF)
	{
	  p.error(ERROR_EOF_IN_BLOCK,"no closing bracket in option list");
	  return 100;
	}
      else if(p.curtok==PT_BBEGIN)
	{
	  p.error(ERROR_USELESS_BLOCK,"Ignoring useless block in option list");
	  dummyread(p);
	}
      else if(p.curtok == PT_TEXT)
	{
	  res=parseItem(p);
	  if(res != 0)
	    return res;
	}
      else
	p.gentoken(); // this should never happen
    };
  p.gentoken();
  return 0;
}

int OptionSet::processMesh(Mesh *mesh, World *world)
{
  if(mesh==NULL)
    return 0;

  if((!create_ncp) && (!create_w))
    {
      delete mesh;
      return 0;
    };

  if(tedelete != 0)
    mesh->deleteTEParts(tedelete);

  if(mesh->isDegenerated())
    {
      delete mesh;
      return 0;
    };
  
  if(transparency != OS_NO_TRANS)
    mesh->setTransparency(transparency);

  if(recolor_blue)
    mesh->setTEColor(bluecolor);

  mesh->setSurfaceType(surface);

  world->addMesh(mesh, create_w, create_ncp);

  return 0;
}
