/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "lookup.h"
#include "ichain.h"

LookupGrid::LookupGrid(rvulong _size)
{
  xlo=zlo=1e40;
  xhi=zhi=-1e40;
  size=_size;
  ichains=NULL;
}

LookupGrid::~LookupGrid(void)
{
  int i;

  if(ichains != NULL)
    {
      for(i=0;i<zsz*xsz;i++)
	delete ichains[i];
      delete ichains;
    };
}

void LookupGrid::expandGrid(Vector &v)
{
  if(v.x() > xhi) xhi=v.x();
  if(v.x() < xlo) xlo=v.x();
  if(v.z() > zhi) zhi=v.z();
  if(v.z() < zlo) zlo=v.z();
}


void LookupGrid::write(FILE *f)
{
  int i;
  fwrite(&data, RV_LOOKUPGRID_SIZE,1,f);
  for(i=0;i<zsz*xsz;i++)
    ichains[i]->write(f);
}

IndexChain *LookupGrid::lichain(int x, int z)
{
  if((x<0)||(z<0)||(x>=xsz)||(z>=zsz))
    {
      printf("LOOKUPGRID::LICHAIN(%d,%d)-PANIC!\n",x,z);
      return NULL;
    };
  return ichains[ x + (z*xsz) ];
}

void LookupGrid::doneSizing(void)
{
  rvfloat size_x1, size_x2, size_z1, size_z2;
  int i;

  size_x1 = (xhi-xlo)/30.0; size_z1= (zhi-zlo)/30.0;
  size_x2 = size_z2 = 1000.0;

  data.raster_size=rint((size_x1+ size_x2+ size_z1+ size_z2) / 4.0);
  data.x0 = xlo;
  data.z0 = zlo;
  
  data.x_size = ceil((xhi-xlo)/data.raster_size);
  data.z_size = ceil((xhi-xlo)/data.raster_size);

  xsz=(int)data.x_size; zsz=(int)data.z_size;

  ichains=new (IndexChain*)[zsz*xsz];
  for(i=0;i<zsz*xsz;i++)
    ichains[i]=new IndexChain(size);
}


int LookupGrid::xgridcoord(rvfloat x)
{
  return (int)floor((x-xlo) / data.raster_size);
}


int LookupGrid::zgridcoord(rvfloat z)
{
  return (int)floor((z-zlo) / data.raster_size);
}
