/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>

#include "rvtypes.h"
#include "polygon.h"


Polygon::Polygon(FILE *f)
{
  fread(&data,RV_POLY_SIZE,1,f);
}


void Polygon::setTransparency(rvulong t)
{
  int i;
  for(i=0;i<numverts();i++)
    data.color[i] = (data.color[i] & 0xffffff) | (t << 24);
  data.type = (data.type & (~0x104)) | 0x004;
}


int Polygon::numverts(void)
{
  return (data.type & 1) ? 4 : 3;
}

void Polygon::write(FILE *f)
{
  fwrite(&data,RV_POLY_SIZE,1,f);
}







bool Polygon::teheur_BlueWall(void)
{
  int i;
  rvulong r,g,b;
  bool isTE=false;

  if(data.texture != -1)
    return false;

  for(i=0;i<numverts();i++)
    {
      r=(data.color[i]&0xff0000) >> 16;
      g=(data.color[i]&0xff00) >> 8;
      b=(data.color[i]&0xff);

      if((r!=g)||(g!=b))
	isTE=true;
    };
  return isTE;
}

int Polygon::vertidx(int idx)
{
  return data.vertidx[idx];
}

void Polygon::setTEColor(rvulong color)
{
  rvulong r,g,b;
  float scale;
  int i;

  if(!teheur_BlueWall())
    return;

  for(i=0;i<numverts();i++)
    {
      r=(data.color[i]&0xff0000) >> 16;
      g=(data.color[i]&0xff00) >> 8;
      b=(data.color[i]&0xff);

      scale = ((float)r + (float)g + (float)b) / 765.0;

      r=(rvulong)(   (float)( (color&0xff0000) >> 16 )    * scale );
      g=(rvulong)(   (float)( (color&0xff00)   >> 8  )    * scale );
      b=(rvulong)(   (float)( (color&0xff)           )    * scale );

      data.color[i]= (data.color[i] & 0xff000000)
	| (r<<16) | (g<<8) | (b);

    };
}


bool Polygon::teheur_Lamp(void)
{
  int i;
  rvulong r,g,b;
  bool isTE=true;

  if(data.texture != -1)
    return false;

  for(i=0;i<numverts();i++)
    {
      r=(data.color[i]&0xff0000) >> 16;
      g=(data.color[i]&0xff00) >> 8;
      b=(data.color[i]&0xff);

      if((r!=g)||(g!=b))
	isTE=false;
    };
  return isTE;
}

bool Polygon::isInTexture(rvshort page,
			  rvfloat ulo, rvfloat uhi,
			  rvfloat vlo, rvfloat vhi)
{
  int i;

  if(data.texture != page)
    return false;

  for(i=0;i<numverts();i++)
    {
      if( data.texcoord[i].u < ulo )
	return false;
      if( data.texcoord[i].v < vlo )
	return false;
      if( data.texcoord[i].u > uhi )
	return false;
      if( data.texcoord[i].v > vhi )
	return false;
    };
  return true;
}


bool Polygon::teheur_Floor(void)
{
  return isInTexture(0,  0.0,0.1  ,  0.9,1.0);
}

bool Polygon::teheur_Box(void)
{
  return isInTexture(7,  0.0,0.75  ,  0.0,0.25);
}

bool Polygon::teheur_CarImage(void)
{
  return isInTexture(7,  0.0,1.0  ,  0.25,0.75);
}


bool Polygon::teheur_RWall(void)
{ 
  if(isInTexture(7, 0.0,1.0, 0.75,1.0))
    return true;
  return isInTexture(7, 0.75,1.0, 0.0,0.25);
}



int Polygon::detTEPart(void)
{
  /* This runs some heuristics to guess what part of a track editor track
     this polygon could be */

  if(teheur_BlueWall())
    return TEP_PWALL;

  if(teheur_Lamp())
    return TEP_LAMP;

  if(teheur_Floor())
    return TEP_FLOOR;

  if(teheur_RWall())
    return TEP_RWALL;

  if(teheur_Box())
    return TEP_BOX;

  if(teheur_CarImage())
    return TEP_CARIMG;

  return TEP_TRACK;
}

