/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "vector.h"
#include "rvtypes.h"

Vector::Vector(Vector &v1, Vector &v2)
{
  *this = v1 * v2;
  normalize();
}

bool   Vector::operator == (Vector &p)
{
  return FEQUAL(x(),p.x()) && FEQUAL(y(),p.y()) && FEQUAL(z(),p.z());
}

bool   Vector::operator != (Vector &p)
{
  return (!FEQUAL(x(),p.x())) || (!FEQUAL(y(),p.y())) || (!FEQUAL(z(),p.z()));
}

rvfloat Vector::scalarprod(Vector &v)
{
  return x()*v.x() + y()*v.y() + z()*v.z();
}

Vector Vector::operator - (Vector &p)
{
  return Vector( x()-p.x(), y()-p.y(), z()-p.z() );
}

Vector Vector::operator + (Vector &p)
{
  return Vector( x()+p.x(), y()+p.y(), z()+p.z() );
}

rvfloat &Vector::x(void)
  { return data.coord[0]; }

rvfloat &Vector::y(void)
  { return data.coord[1]; }

rvfloat &Vector::z(void)
  { return data.coord[2]; }

rvfloat Vector::x(rvfloat f)
  { return x()=f; }

rvfloat Vector::y(rvfloat f)
  { return y()=f; }

rvfloat Vector::z(rvfloat f)
  { return z()=f; }




Vector::Vector()
{
  data.coord[0]=data.coord[1]=data.coord[2]=0.0;
}
  
Vector::Vector(rvfloat _x, rvfloat _y, rvfloat _z)
{
  x(_x); y(_y); z(_z); 
}

Vector::Vector(RV_Vector *mem)
{
  memcpy(&data,mem,RV_VECTOR_SIZE);
}

Vector::Vector(const Vector &p)
{
  data=p.data;
}

rvfloat Vector::length(void)
{
  return sqrt( SQR(x()) + SQR(y()) + SQR(z()) );
}

bool Vector::normalize(void)
{
  rvfloat lgt=length();
  
  if(lgt < EPSILON)
    return false;
  x() /= lgt;
  y() /= lgt;
  z() /= lgt;
  return true;
}

Vector Vector::operator * (rvfloat scalefac)
{
  return Vector(x()*scalefac, y()*scalefac, z()*scalefac );
}

Vector Vector::operator * (Vector &p)
{
  return Vector(
		y()*p.z() - z()*p.y(),
		z()*p.x() - x()*p.z(),
		x()*p.y() - y()*p.x() );
}
