/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>

#include "rvtypes.h"
#include "world.h"
#include "wwriter.h"
#include "ncpwriter.h"

#define MAX_MESHES 10000

World::World()
{
  mlist=new (Mesh*)[MAX_MESHES];
  cw=new bool[MAX_MESHES];
  cncp=new bool[MAX_MESHES];
  
  cur_size=0;
} 

World::~World()
{
  int i;
  
  for(i=0;i<cur_size;i++)
    delete mlist[i];

  delete mlist;
  delete cw;
  delete cncp;
}



bool World::addMesh(Mesh *m, bool create_w, bool create_ncp)
{
  if(cur_size < MAX_MESHES)
    {
      mlist[cur_size]=m;
      cw[cur_size]=create_w;
      cncp[cur_size]=create_ncp;
      cur_size++;
      return true;
    }
  else
    return false;
}

void showPercent(int now, int total, int &last)
{
  int act=(now*100)/total;

  if(act > last)
    {
      printf("\r  %3d %%   ",act); fflush(stdout);
    };
}

int World::write(const char *wname, const char *ncpname, bool showStatus)
{
  int i;
  int wcnt=0;
  int polycnt=0;
  int last;

  if(wname != NULL)
    {
      WFileWriter wf(cur_size);
      
      if(showStatus)
	printf("Creating W File\n");
      for(i=0;i<cur_size;i++)
	if(cw[i])
	  {
	    wf.addMesh(mlist[i]);
	    wcnt++;
	  };
      if(showStatus)
	printf("Writing W File '%s'\n",wname);

      wf.write(wname);
    };

  if(ncpname != NULL)
    {
      for(i=0;i<cur_size;i++)
	if(cncp[i])
	  polycnt += mlist[i]->numPolys();

      NCPFileWriter ncpf(polycnt);
      last=0;

      if(showStatus)
	printf("Creating NCP Data\n");

      for(i=0;i<cur_size;i++)
	if(cncp[i])
	  {
	    ncpf.addMesh(mlist[i]);
	    //	    if(showStatus)
	    //showPercent(i,cur_size,last);
	  };

      if(showStatus)
	printf("Creating NCP Grid\n");
      ncpf.createGrid();
      
      if(showStatus)
	printf("Writing NCP File '%s'\n",ncpname);
	
      ncpf.write(ncpname);
    };
  return wcnt;
}

