/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <math.h>

#include "mesh.h"
#include "wwriter.h"


WFileWriter::WFileWriter(rvulong maxsz)
{
  cur_size=0;
  max_size=maxsz;
  mlist=new (Mesh*)[maxsz];
}

WFileWriter::~WFileWriter()
{
  delete mlist;
}

bool WFileWriter::addMesh(Mesh *m)
{
  if(cur_size < max_size)
    {
      mlist[cur_size]=m;
      cur_size++;
      return true;
    }
  else
    return false;
}

void WFileWriter::putFunnyBall(FILE *f)
{
  /* THIS IS VERY ANNOYING: JUST ONE FUNNYBALL. */
  rvfloat fl;
  rvulong ul;

  // One ball
  ul=1; fwrite(&ul,sizeof(rvulong),1,f);

  // Center <0,0,0>
  fl=0.0; fwrite(&fl,sizeof(rvfloat),1,f);
  fl=0.0; fwrite(&fl,sizeof(rvfloat),1,f);
  fl=0.0; fwrite(&fl,sizeof(rvfloat),1,f);

  // Radius 100'000 should be enough
  fl=100000.0; fwrite(&fl,sizeof(rvfloat),1,f);
 

  // And everything inside it:
  fwrite(&cur_size,sizeof(rvulong),1,f);

  for(ul=0;ul<cur_size;ul++)
    fwrite(&ul,sizeof(rvulong),1,f);

}

int WFileWriter::write(const char *fname)
{
  FILE *f;
  rvulong i;

  f=fopen(fname,"wb");
  if(f==NULL)
    return errno;

  fwrite(&cur_size, sizeof(rvulong),1,f);
  for(i=0;i<cur_size;i++)
    mlist[i]->write_w(f);

  putFunnyBall(f);

  fclose(f);
  return 0;
}

